# =============================================================================================================
# directory listing related functions
# =============================================================================================================

use strict;
use warnings;

#--------------------------------------------------------------------------------------------------------------
# List Directory
#--------------------------------------------------------------------------------------------------------------

sub ls_dir 
{
	&plog(3, "sub ls_dir");
	&prep_globals;
	
	if($main::LISTING)
	{
		&plog(0, "sub ls_dir: Allready preforming a list, aborting new list attempt");
		return 0;
	}

	$main::STOP = 0;
	$main::LISTING = 1;
	$main::hl_counter = 0;
	$main::hlist2->delete("all");
	chdir $main::dir;	# shift directory, not just list ;)
	$main::dir = cwd(); 

        my @file_list = ();
        my @dirlist = &dir_filtered($main::dir);

	&ls_dir_print("..");

        if($main::filter_use_re == 0 && $main::filter_string ne "") 
        {
		&plog(4, "sub ls_dir: escaping \$main::filter_string");
        	$main::filter_string_escaped = &escape_string($main::filter_string);
        }
        else 
        {
        	$main::filter_string_escaped = $main::filter_string;
        }

        if($main::recr) 
        {
		&plog(4, "sub ls_dir: recursive mode");
	        find(\&ls_dir_print_findpassthru, "$main::dir");
                $main::LISTING = 0;
                $main::FIRST_DIR_LISTED = 0;
	        return;
        }
        else 
        {
		&plog(4, "sub ls_dir: non recursive mode");
        	for(@dirlist) 
        	{
                	if(-d $_) 
                	{
	                	&ls_dir_print($_);	# print directorys 1st
                        }
                        else 
                        {
                        	push @file_list, $_;	# push files to array
                        }
                }
                for(@file_list) 
                {
                	if($main::STOP == 1)
			{
				return 0;
			}
                	&ls_dir_print($_);		# then print the file array after all dirs have been printed
                }
                $main::LISTING = 0;
                $main::FIRST_DIR_LISTED = 0;
                return;
        }
}

#--------------------------------------------------------------------------------------------------------------
# ls_dir_print
#--------------------------------------------------------------------------------------------------------------

# little bypass for using find, as perl find stubbornly refuses to pass vars in any other way than $_

sub ls_dir_print_findpassthru
{
	if(&match_filter($_) | -d $_)
	{
		&ls_dir_print($_);
	}
}

# this function is only called from ls_dir

sub ls_dir_print
{
	my $file = shift;

	if($main::STOP == 1)
	{
		&plog(4, "sub ls_dir_print: \$main::STOP = \"$main::STOP\" - not printing");
		return 0;
	}
	&plog(3, "sub ls_dir_print: \"$file\"");
	
	$main::hlist_cwd = cwd;
	
        my $tag = "";
        my $art = "";
        my $tit = "";
        my $tra = "";
        my $alb = "";
        my $com = "";
        my $gen = "";
        my $year = "";

        if(!$file || $file eq "" || $file eq ".") 
        {
		&plog(4, "sub ls_dir_print: \$file = \"$file\" not printing");
        	return;
        }

	# when doing recursive, pad new dir with a blank line
	if(-d $file && $main::recr) 	
	{
		&plog(4, "sub ls_dir_print: checking to see if a blank line should be inserted for directory padding");
 		if($main::FIRST_DIR_LISTED) 
 		{
			&plog(4, "sub ls_dir_print: First directory has been listed, padding");
                	&nf_print("", "");
                }
                else
                {
			&plog(4, "sub ls_dir_print: First directory has not been listed, not padding");
			$main::FIRST_DIR_LISTED = 1;
                }
		&nf_print($file, $file);
		return;
	}

	# if is mp3 & id3_mode enabled then grab id3 tags
	if($file =~ /.*\.mp3$/i && $main::id3_mode == 1) 
	{
		&plog(4, "sub ls_dir_print: if is mp3 & id3_mode enabled then grab id3 tags");
		($tag, $art, $tit, $tra, $alb, $gen, $year, $com) = &get_tags($file);

       		if ($tag eq "id3v1") 
       		{
			&plog(4, "sub ls_dir_print: got id3 tags, printing");
			&nf_print($file, $file, $art, $tit, $tra, $alb, $com, $gen, $year);
			return;
		}
	}

	&plog(4, "sub ls_dir_print: doesnt meet any special conditions, just print it");
	&nf_print($file, $file);
}

#--------------------------------------------------------------------------------------------------------------
# Dir Dialog
#--------------------------------------------------------------------------------------------------------------

sub dir_dialog 
{
	&plog(3, "sub dir_dialog");
	my $old_dir = $main::dir;

	my $dd_dir = $main::mw->chooseDirectory
	(
		-initialdir=>$main::dir,
		-title=>"Choose a directory"
	);

	if($dd_dir) 
	{
		$main::dir = $dd_dir;
                chdir $main::dir;
		&ls_dir;
	}
	else 
	{
		$main::dir = $old_dir;
	}
}

#--------------------------------------------------------------------------------------------------------------
# fn_readdir, also removes . and .. from listing which nf needs
#--------------------------------------------------------------------------------------------------------------

sub fn_readdir
{
	&plog(3, "sub fn_readdir");
	my $dir = shift;
	my @dirlist = ();
	my @d = ();

	opendir(DIR, "$dir") or die "can't open dir \"$dir\": $!";
	@dirlist = CORE::readdir(DIR);
	closedir DIR;
					# -- make sure we dont have . and .. in array --
	@dirlist = reverse @dirlist;    # this puts . and .. and end of array
	$#dirlist = $#dirlist - 2;      # remove . and ..
	@dirlist = &ci_sort(@dirlist);  # sort array
	return @dirlist;
}

sub dir_filtered
{
	&plog(3, "sub dir_filtered");
	my $dir = shift;

	my @d = ();
	my @dirlist = &fn_readdir($dir);
	
	for my $i(@dirlist)
	{
		if(-d $i && (!$main::LISTING && !$main::proc_dirs))	# ta tibbsy for help
 		{
			&plog(4, "sub dir_filtered: $i is dir, didnt pass");
 			next;
 		}
		if(&match_filter($i) == 1)		# apply listing filter
		{	
			&plog(4, "sub dir_filtered: $i passed");
			push @d, $i;
		}
		else
		{
			&plog(4, "sub dir_filtered: $i didnt pass");
		}
	}
	return @d;
}

1;